require 'uri'
require 'json'
require 'httparty'

class DiscoClient
	include HTTParty

	headers 'Accept' => 'application/json'
	headers 'Content-Type' => 'application/json'

	DEFAULT_CONTAINER = {
		id: ENV['CONTAINER_NAME'],
		type: 'http',
		port: 3000
	}

	def initialize(options = {})
		@base_uri = options[:base_uri] || ENV['DISCO_BASE_URI']
	end

	def register(service_id, options = {})
		body = DEFAULT_CONTAINER.merge(options)
		body[:service] = { id: service_id };

		@container = request('post', '1/containers', nil, body)
	end

	def get_services
		request('get', '1/services')
	end

	def get_service(id)
		URI.encode_www_form_component(id)
		request('get', "1/services/#{id}")
	end

	def get_container(id = nil)
		id = URI.encode_www_form_component(id || @container[:id])
		request('get', "1/containers/#{id}")
	end

	def get_configuration(id = nil)
		id = URI.encode_www_form_component(id || @container[:id])
		request('get', "1/containers/#{id}/configuration")
	end

	private

	def request(method, path, params = nil, body = nil)
		uri = URI.join(@base_uri, path)

		options = {}
		options[:query] = params if params
		options[:body] = JSON.generate(body) if body

		res = self.class.send(method, uri, options)

		# 1xx should be handled by httparty

		# 2xx - success
		if res.code < 300
			return nil unless res.body && res.body.length
			return JSON.parse(res.body, symbolize_names: true, create_additions: false)
		end

		# 3xx redirections are handled by httparty
		# 304 is raised as an error

		res.error!
	end
end
